{% extends "layout.jinja2" %}

{% block content %}
<div class="row">
  <div class="col-md-6">
    <p class="pull-right"><a href="{{request.route_url('c2cgeoform_index')}}">{{_("Back to overview")}}</a></p>
    <h3>{{_("Edit")}}</h3>
    {{form|safe}}
  </div>
</div>
{% endblock content %}
